/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.composer.node.collection;

import java.util.Iterator;

import shohaku.composer.CompositeException;
import shohaku.composer.Node;
import shohaku.composer.node.AbstractValueNode;
import shohaku.core.collections.Group;
import shohaku.core.collections.GroupUtils;
import shohaku.core.collections.group.LinkedHashListGroup;
import shohaku.core.lang.ObjectCreationException;
import shohaku.core.lang.ResourceLoader;

/**
 * グループ型のデータを生成する値型のノードを提供します。
 */
public class GroupNode extends AbstractValueNode {

    /**
     * 値を生成して返却します。
     * 
     * @return 生成された値
     */
    protected Object generateValue() {

        String type = getAttribute("type", LinkedHashListGroup.class.getName());
        Group group;
        try {
            group = (Group) ResourceLoader.getInstance(type, getClassLoader());
        } catch (ClassCastException e) {
            throw new CompositeException("class name which is not a Group class is specified. " + type, e);
        } catch (ObjectCreationException e) {
            throw new CompositeException("illegal class name is specified. " + type, e);
        }

        for (Iterator i = getNodeContext().valueElementIterator(); i.hasNext();) {
            Node arg = (Node) i.next();
            group.addAll((Group) arg.getNodeValue());
        }

        return GroupUtils.unmodifiableGroup(group);
    }

}
